import { BaseLoad } from '@kevisual/load';

type GlobalContext<U = any> = {
  redis?: any;
} & U;

export const useContext = <T = GlobalContext>(initContext?: GlobalContext): T => {
  const context: GlobalContext = (global as any).context;
  const _context = context || initContext;
  if (!context) {
    if (_context) {
      (global as any).context = _context;
    } else {
      global.context = {};
    }
  }
  return global.context;
};

/**
 * 获取上下文，当key存在时，返回key对应的值，不存在时，返回init函数的值
 * 若init不存在，key存在，直接进行循环检测，直到key存在，需要await
 *
 * @param key
 * @param init
 * @returns
 */
export const useContextKey = <T = any>(key: string, init?: () => T): T => {
  const _context = useContext({});
  if (key && typeof _context[key] !== 'undefined') {
    return _context[key];
  }
  if (key && init) {
    // 没有初始化过，初始化函数
    _context[key] = (init as () => T)();
    return _context[key] as any;
  }
  // 其他情况，默认已经初始化过
  if (key) {
    // 加载
    const baseLoad = new BaseLoad();
    const voidFn = async () => {
      return _context[key];
    };
    const checkFn = async () => {
      const loadRes = await baseLoad.load(voidFn, {
        key,
        isReRun: true,
        checkSuccess: () => _context[key],
        timeout: 5 * 60 * 1000,
        interval: 1000,
        //
      });
      if (loadRes.code !== 200) {
        console.error('load key error');
        return null;
      }
      return _context[key];
    };
    return checkFn() as any;
  }
  console.error('key is empty');
  return null;
};

export const setContextKey = <T>(key: string, value: T) => {
  const _context = useContext<GlobalContext<T>>({});
  _context[key] = value;
  return _context;
};
